#include <zephyr/zephyr.h>
#include <gtk/gtk.h>
#include <time.h>
#include "gtkzephyr.h"
#include "chatdialog.h"
#include "zcalls.h"
#include "errorbox.h"



void gtkzephyr_chat_handle_servack(ZNotice_t *notice, gtkzephyr_userstruct *user) {

  struct tm *senttime = localtime(&(notice->z_time.tv_sec));
  gchar *line, *header, *message;
  gchar **lines;
  gint position, i;

  if (notice->z_kind == SERVNAK) {
    // Not sure what to do here
  } else if (!strcmp(notice->z_message, ZSRVACK_SENT)) {
    /* Do nothing special.. message always gets displayed anyways. */
  } else if (!strcmp(notice->z_message, ZSRVACK_NOTSENT)) {

    gchar *msg = "*** ERROR: User not logged in - following message NOT sent **\n";
    gtk_editable_set_position(GTK_EDITABLE(user->chattext),
	gtk_text_get_length(GTK_TEXT(user->chattext)));
    position = gtk_editable_get_position(GTK_EDITABLE(user->chattext));
    gtk_editable_insert_text(GTK_EDITABLE(user->chattext), msg, strlen(msg),
	&position);

  } else {

    gtkzephyr_error("Unknown server response");
    gtkzephyr_handle_unknown_notice(notice);

  }
  
  header = g_strdup_printf("[%.2d:%.2d] <%s> ", senttime->tm_hour,
      senttime->tm_min, gtkzephyr_get_userid());
  message = gtk_editable_get_chars(GTK_EDITABLE(user->chatsendentry), 0, -1);
  lines = g_strsplit(message, "\n", 0);
  gtk_text_freeze(GTK_TEXT(user->chattext));
  gtk_editable_set_position(GTK_EDITABLE(user->chattext),
      gtk_text_get_length(GTK_TEXT(user->chattext)));
  position = gtk_editable_get_position(GTK_EDITABLE(user->chattext));
  for (i = 0; lines[i] != NULL; ++i) {
    line = g_strdup_printf("%s %s\n", header, lines[i]);
    gtk_editable_insert_text(GTK_EDITABLE(user->chattext), line,
				strlen(line), &position);
    g_free(line);
    if (i == 0) {
			gchar *tmp = g_strnfill(strlen(header), ' ');
			g_free(header);
			header = tmp;
		}
  }
  gtk_text_thaw(GTK_TEXT(user->chattext));
  gtk_adjustment_set_value(GTK_TEXT(user->chattext)->vadj,
      GTK_TEXT(user->chattext)->vadj->upper);
  g_free(header);
  g_free(message);
  g_strfreev(lines);

  gtk_editable_delete_text(GTK_EDITABLE(user->chatsendentry), 0, -1);
  gtk_widget_set_sensitive(user->chatsendentry, TRUE);
  gtk_widget_set_sensitive(user->chatsendbutton, TRUE);
  gtk_widget_grab_focus(user->chatsendentry);

}



void gtkzephyr_chat_handle_message(ZNotice_t *notice, gtkzephyr_userstruct *user) {

  struct tm *senttime = localtime(&(notice->z_time.tv_sec));
  gchar *line, *header, *message;
  gchar **lines;
  gint position, i;

  header = g_strdup_printf("[%.2d:%.2d] <%s> ", senttime->tm_hour,
      senttime->tm_min, user->userid);
  message = gtkzephyr_get_body(notice->z_message, notice->z_message_len);
  lines = g_strsplit(message, "\n", 0);
  gtk_text_freeze(GTK_TEXT(user->chattext));
  gtk_editable_set_position(GTK_EDITABLE(user->chattext),
      gtk_text_get_length(GTK_TEXT(user->chattext)));
  position = gtk_editable_get_position(GTK_EDITABLE(user->chattext));
  for (i = 0; lines[i] != NULL; ++i) {
    line = g_strdup_printf("%s %s\n", header, lines[i]);
    gtk_editable_insert_text(GTK_EDITABLE(user->chattext), line,
	strlen(line), &position);
    g_free(line);
    if (i == 0) {
			gchar *tmp = g_strnfill(strlen(header), ' ');
			g_free(header);
			header = tmp;
		}
  }
  gtk_text_thaw(GTK_TEXT(user->chattext));
  gtk_adjustment_set_value(GTK_TEXT(user->chattext)->vadj,
      GTK_TEXT(user->chattext)->vadj->upper);
  g_free(header);
  g_free(message);
  g_strfreev(lines);
  ZFreeNotice(notice);

}



void close_chat_window(GtkObject *widget, gtkzephyr_userstruct *user) {

  gtk_widget_hide(user->chatwindow);
  gtk_editable_delete_text(GTK_EDITABLE(user->chattext), 0, -1);
  gtk_editable_delete_text(GTK_EDITABLE(user->chatsendentry), 0, -1);
  gtk_widget_set_sensitive(user->chatsendentry, TRUE);
  gtk_widget_set_sensitive(user->chatsendbutton, TRUE);

}



gboolean chatwindow_delete_event(GtkObject *widget, GdkEvent *event, gtkzephyr_userstruct *user) {

  close_chat_window(NULL, user);

  return TRUE;

}



void chat_send_text(GtkEditable *entry, gpointer data) {

  gtkzephyr_userstruct *user = (gtkzephyr_userstruct *) data;
  gtk_widget_set_sensitive(GTK_WIDGET(entry), FALSE);
  gtk_widget_set_sensitive(user->chatsendbutton, FALSE);

  gtkzephyr_send_text(user, entry, gtkzephyr_chat_handle_servack);

}



void make_chat_window(gtkzephyr_userstruct *user) {

  GtkWidget *vbox, *hbuttonbox, *scrolledwindow;

  g_assert(user->chatwindow == NULL);
  g_assert(user->chattext == NULL);
  g_assert(user->chatsendentry == NULL);
  g_assert(user->chatsendbutton == NULL);

  DEBUG_INFO("Chat Window opened for: %s\n", user->userid);

  user->chatwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize(user->chatwindow, 507, 260);
  gtk_signal_connect(GTK_OBJECT(user->chatwindow), "delete_event",
      GTK_SIGNAL_FUNC(chatwindow_delete_event), user);
  gtk_window_set_title(GTK_WINDOW(user->chatwindow), user->userid);
  gtk_container_set_border_width(GTK_CONTAINER(user->chatwindow), 5);
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(user->chatwindow), vbox);
  gtk_widget_show(vbox);
  user->chattext = gtk_text_new(NULL, NULL);
  gtk_text_set_editable(GTK_TEXT(user->chattext), FALSE);
  gtk_widget_show(user->chattext);
  scrolledwindow = gtk_scrolled_window_new(GTK_TEXT(user->chattext)->hadj,
      GTK_TEXT(user->chattext)->vadj);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), user->chattext);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
  gtk_widget_show(scrolledwindow);
  user->chatsendentry = gtk_entry_new();
  gtk_editable_set_editable(GTK_EDITABLE(user->chatsendentry), TRUE);
  gtk_signal_connect(GTK_OBJECT(user->chatsendentry), "activate",
      GTK_SIGNAL_FUNC(chat_send_text), user);
  gtk_box_pack_start(GTK_BOX(vbox), user->chatsendentry, FALSE, FALSE, 3);
  gtk_widget_show(user->chatsendentry);
  hbuttonbox = gtk_hbutton_box_new();
  gtk_button_box_set_child_size(GTK_BUTTON_BOX(hbuttonbox), 65, 15);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbuttonbox), 25);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
  user->chatsendbutton = gtk_button_new_with_label("Close");
  gtk_signal_connect(GTK_OBJECT(user->chatsendbutton), "clicked",
      GTK_SIGNAL_FUNC(close_chat_window), user);
  gtk_widget_show(user->chatsendbutton);
  gtk_widget_show(hbuttonbox);
  gtk_box_pack_end(GTK_BOX(hbuttonbox), user->chatsendbutton, FALSE, TRUE, 25);
  gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, FALSE, FALSE, 5);
}



void chat_remove_unreadmsg(ZNotice_t *notice, gtkzephyr_userstruct *user) {

	gtkzephyr_remove_unreadmsg(user);

}



void gtkzephyr_chat_window(gpointer data) {

  gtkzephyr_userstruct *user = (gtkzephyr_userstruct *) data;

  if (user->chatwindow == NULL)
    make_chat_window(user);

	/* If the userdialog is open, it thinks it has full access to all unread
	 * messages, so we don't mess with them here */
  if (user->unreadmsg && (!user->userdialog || !GTK_WIDGET_VISIBLE(user->userdialog))) {
    g_list_foreach(user->newmsgs, (GFunc)gtkzephyr_chat_handle_message, user);
		g_list_foreach(user->newmsgs, (GFunc)chat_remove_unreadmsg, user);
		g_list_free(user->newmsgs);
		user->newmsgs = NULL;
  }

  gtk_widget_show(user->chatwindow);
  gdk_window_raise(user->chatwindow->window);

}


void gtkzephyr_open_chat_window(GtkWidget *widget, gpointer data) {

  gtkzephyr_chat_window(data);

}
