#include <zephyr/zephyr.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include "prefs.h"
#include "gtkzephyr.h"
#include "userdialog.h"
#include "chatdialog.h"
#include "zcalls.h"
#include "adduser.h"

GList *userlist;
GHashTable *usertable;
GtkWidget *mainwindow, *mainclist, *statusbar, *msgcounter;
gint redraw_count;
gint newmsg_count;
guint msgcounter_context;


gtkzephyr_userstruct *gtkzephyr_get_userstruct(gchar *userrealm) {
  DEBUG_INFO("Looking for %s..\n", userrealm);
  return (gtkzephyr_userstruct *) g_hash_table_lookup(usertable, userrealm);
}


void gtkzephyr_add_unreadmsg(gtkzephyr_userstruct *user) {

  gint rownumber = gtk_clist_find_row_from_data(GTK_CLIST(mainclist), user);
  gchar *unreadmsg_str = g_strdup_printf("%d", ++user->unreadmsg);
  gchar *msgcounter_str = g_strdup_printf("%d Unread Msgs", ++newmsg_count);
  
  if (rownumber == -1) {
    append_to_clist(user, NULL);
  } else {
    gtk_clist_set_text(GTK_CLIST(mainclist), rownumber, 0, unreadmsg_str);
  }
  g_free(unreadmsg_str);

  gtk_statusbar_push(GTK_STATUSBAR(msgcounter), msgcounter_context,
      msgcounter_str);
  g_free(msgcounter_str);

  if (redraw_count < prefstruct->max_redraw) {
    gtk_clist_sort(GTK_CLIST(mainclist));
    if (++redraw_count == prefstruct->max_redraw)
      gtk_clist_freeze(GTK_CLIST(mainclist));
  }
}



void gtkzephyr_remove_unreadmsg(gtkzephyr_userstruct *user) {

  gint rownumber = gtk_clist_find_row_from_data(GTK_CLIST(mainclist), user);
  gchar *unreadmsg_str = g_strdup_printf("%d", --user->unreadmsg);

  g_assert(user->unreadmsg >= 0);
  g_assert(rownumber >= 0);
  if (user->unreadmsg) {
    DEBUG_INFO("Setting text to %s\n", unreadmsg_str);
    gtk_clist_set_text(GTK_CLIST(mainclist), rownumber, 0,
	user->unreadmsg ? unreadmsg_str : "");
  } else {
    DEBUG_INFO("Setting null text\n");
    if (user->status || prefstruct->show_disconnected) {
      DEBUG_INFO("Setting null text\n");
      gtk_clist_set_text(GTK_CLIST(mainclist), rownumber, 0, "");
    } else {
      DEBUG_INFO("Removing user from clist\n");
      gtk_clist_remove(GTK_CLIST(mainclist), rownumber);
    }
  }
  g_free(unreadmsg_str);

  newmsg_count--;
  g_assert(newmsg_count >= 0);
  gtk_statusbar_pop(GTK_STATUSBAR(msgcounter), msgcounter_context);


  if (redraw_count < prefstruct->max_redraw) {
    gtk_clist_sort(GTK_CLIST(mainclist));
    if (++redraw_count == prefstruct->max_redraw)
      gtk_clist_freeze(GTK_CLIST(mainclist));
  }
}



void gtkzephyr_handle_user_login(gtkzephyr_userstruct *user) {

  if (!prefstruct->show_disconnected) {
    DEBUG_INFO("Adding %s to mainclist..\n", user->userid);
    append_to_clist(user, NULL);
  }
  user->status = user->oldstatus = 1;

}



void gtkzephyr_handle_user_logout(gtkzephyr_userstruct *user) {

	if (!user->unreadmsg) {
		if (!prefstruct->show_disconnected) {
			DEBUG_INFO("Removing %s from mainclist..\n", user->userid);
			gtk_clist_remove(GTK_CLIST(mainclist), gtk_clist_find_row_from_data(
				GTK_CLIST(mainclist), user));
		}
		user->oldstatus = 0;
	}

	user->status = 0;

}
    



void gtkzephyr_add_user(gchar *userid) {
  FILE *fp;
  gchar *filename;
	gtkzephyr_userstruct *user;

  DEBUG_INFO("Adding new user to file..\n");

  filename = g_strconcat(getenv("HOME"), "/.anyone", NULL);
  DEBUG_INFO("Filename: %s\n", filename);
  fp = fopen(filename, "a");
  fputs(userid, fp);
  fputs("\n", fp);
  fclose(fp);
  g_free(filename);

	gtkzephyr_make_userstruct(userid);
	update_userlist(NULL);
}



gint userlist_comparefunc(gconstpointer a, gconstpointer b) {

  gtkzephyr_userstruct *usera = (gtkzephyr_userstruct *)a;
  gtkzephyr_userstruct *userb = (gtkzephyr_userstruct *)b;

  if (usera->unreadmsg == 0 && userb->unreadmsg == 0) {
    if (usera->status < userb->status) {
      return 1;
    } else {
      return -1;
    }
  } else {
    if (usera->unreadmsg < userb->unreadmsg) {
      return 1;
    } else {
      return -1;
    }
  }
}

gint clist_comparefunc(GtkCList *clist, gconstpointer a, gconstpointer b) {

  gtkzephyr_userstruct *usera = (gtkzephyr_userstruct*)((GtkCListRow *)a)->data;
  gtkzephyr_userstruct *userb = (gtkzephyr_userstruct*)((GtkCListRow *)b)->data;

  if (usera->unreadmsg == 0 && userb->unreadmsg == 0) {
    if (usera->status < userb->status) {
      return 1;
    } else {
      return -1;
    }
  } else {
    if (usera->unreadmsg < userb->unreadmsg) {
      return 1;
    } else {
      return -1;
    }
  }
}



#define unsensitive_menu_item(x)  menu_item = gtk_menu_item_new_with_label(x); \
                                  gtk_menu_append(GTK_MENU(menu), menu_item); \
                                  gtk_widget_set_sensitive(menu_item, FALSE); \
                                  gtk_widget_show(menu_item);
#define menu_item(label, cb) menu_item = gtk_menu_item_new_with_label(label); \
                             gtk_menu_append(GTK_MENU(menu), menu_item); \
                             gtk_signal_connect(GTK_OBJECT(menu_item), "activate", \
                             GTK_SIGNAL_FUNC(cb), NULL); \
                             gtk_widget_show(menu_item);

#define menu_item2(label, cb, pointer) menu_item = gtk_menu_item_new_with_label(label); \
                                       gtk_menu_append(GTK_MENU(menu), menu_item); \
                                       gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cb), pointer); \
                                       gtk_widget_show(menu_item);

void build_menus(GtkMenuBar *menubar) {

  GtkWidget *menu, *menu_item;

  menu = gtk_menu_new();
  unsensitive_menu_item("Send Zephyr");
  menu_item("Add User", gtkzephyr_add_user_dialog);
  unsensitive_menu_item("Preferences");
  menu_item("Exit", gtk_main_quit);

  menu_item = gtk_menu_item_new_with_label("Program");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menubar), menu_item);
  gtk_widget_show(menu_item);

  menu = gtk_menu_new();
  menu_item("Flush locations", gtkzephyr_flush_loc);
  menu_item("Hide", gtkzephyr_hide);
  menu_item("Unhide", gtkzephyr_unhide);
  unsensitive_menu_item("Subscribe");
  unsensitive_menu_item("Unubscribe");

  menu_item = gtk_menu_item_new_with_label("Zctl");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menubar), menu_item);
  gtk_widget_show(menu_item);
  
}



GtkWidget *make_user_menu(gtkzephyr_userstruct *user) {

  GtkWidget *menu, *menu_item;

  menu = gtk_menu_new();
  menu_item2("Read Message",gtkzephyr_open_read,user);
  menu_item2("Send Message", gtkzephyr_open_write,user);
  menu_item2("Open Chat", gtkzephyr_open_chat_window,user);
  menu_item2("Get Info", gtkzephyr_open_info, user);
  unsensitive_menu_item("Remove User");

  return menu;
}



void mainclist_row_selected(GtkWidget *clist, GdkEventButton *event, gpointer data) {
  GdkEventButton *event_button = (GdkEventButton *) event;
  gint row, column;
  gtkzephyr_userstruct *user;

  gtk_clist_get_selection_info(GTK_CLIST(mainclist), event->x, event->y, &row,
      &column);
  gtk_clist_select_row(GTK_CLIST(mainclist),row,column);
  user = gtk_clist_get_row_data(GTK_CLIST(mainclist),GPOINTER_TO_INT(row));
  if (event->type == GDK_BUTTON_PRESS && event_button->button == 3) {
    if (user->menu == NULL)
      user->menu = make_user_menu(user);
    gtk_menu_popup(GTK_MENU(user->menu), NULL, NULL, NULL, NULL,
	event_button->button, event_button->time);
  } else if (event->type == GDK_2BUTTON_PRESS && event_button->button == 1) {
      /* user dialog */
    gtkzephyr_user_dialog(user);
  } else if (event->type == GDK_2BUTTON_PRESS && event_button->button == 2) {
      /* chat window */
    gtkzephyr_chat_window(user);
  }
}


void build_main_window() {

  GtkWidget *vbox, *menubar, *scrolledwindow;

  mainwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_default_size(GTK_WINDOW(mainwindow), 125, 300);
  gtk_signal_connect(GTK_OBJECT(mainwindow), "delete_event",
      (GtkSignalFunc) gtk_main_quit, NULL);
  gtk_window_set_title(GTK_WINDOW(mainwindow), "GtkZephyr");
  gtk_container_set_border_width(GTK_CONTAINER(mainwindow), 5);
  
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(mainwindow), vbox);
  gtk_widget_show(vbox);

  menubar = gtk_menu_bar_new();
  gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 3);
  gtk_widget_show(menubar);

  build_menus(GTK_MENU_BAR(menubar));

  mainclist = gtk_clist_new(2);
  gtk_clist_set_column_width(GTK_CLIST(mainclist),0,8);
  gtk_clist_set_column_title(GTK_CLIST(mainclist),1, "Userid");
  gtk_clist_column_titles_show(GTK_CLIST(mainclist));
  gtk_clist_set_compare_func(GTK_CLIST(mainclist), clist_comparefunc);
  gtk_signal_connect(GTK_OBJECT(mainclist), "button_press_event",
                     GTK_SIGNAL_FUNC(mainclist_row_selected), NULL);

  scrolledwindow = gtk_scrolled_window_new(GTK_CLIST(mainclist)->hadjustment,
                                           GTK_CLIST(mainclist)->vadjustment);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 3);
  gtk_widget_show(scrolledwindow);

  gtk_container_add(GTK_CONTAINER(scrolledwindow), mainclist);
  gtk_widget_show(mainclist);

  msgcounter = gtk_statusbar_new(); 
  gtk_box_pack_start(GTK_BOX(vbox), msgcounter, FALSE, FALSE, 3);
  msgcounter_context = gtk_statusbar_get_context_id(GTK_STATUSBAR(msgcounter), "Message Counter");
  gtk_statusbar_push(GTK_STATUSBAR(msgcounter), msgcounter_context, "No New Msgs");
  gtk_widget_show(msgcounter);

  statusbar = gtk_statusbar_new();
  gtk_box_pack_start(GTK_BOX(vbox), statusbar, FALSE, FALSE, 0);
  gtk_widget_show(statusbar);

}



gtkzephyr_userstruct *gtkzephyr_make_userstruct(gchar *userid) {

    gtkzephyr_userstruct *user = g_new(gtkzephyr_userstruct,1);
    user->userid = g_strdup(userid);
    user->userrealm = g_strdup_printf("%s@%s", userid, ZGetRealm());
    DEBUG_INFO("userrealm: %s\n", user->userrealm);
    user->newmsgs = NULL;
    user->readmsgs = NULL;
    user->unreadmsg = 0;
    user->status = 0;
    user->oldstatus = 0;
    user->menu = NULL;
    user->userdialog = NULL;
    user->notebook = NULL;
    user->newmsgclist = NULL;
    user->newmsgtext = NULL;
    user->dialogsendtext = NULL;
    user->dialogsendbutton = NULL;
    user->chatwindow = NULL;
    user->chattext = NULL;
    user->chatsendentry = NULL;
    user->chatsendbutton = NULL;
    userlist = g_list_append(userlist,user);
    g_hash_table_insert(usertable, user->userrealm, user);

    gtkzephyr_sub_login_class(user);

		return user;
}



void build_user_list() {

  register char *cp;
  FILE *fp;
  gchar *filename, userid[BUFSIZ];

  usertable = g_hash_table_new(g_str_hash, g_str_equal);
  filename = g_strconcat(getenv("HOME"), "/.anyone", NULL);
  fp = fopen(filename, "r");
	if (fp) {

		for (;;) {
			if (!fgets(userid,sizeof(userid) + 1,fp))
				break;
			if (userid[0] == '\0' || userid[0] == '\n' || userid[0] == '#')
				continue;
			if ((cp = strchr(userid, '#')))
				*cp = '\0'; /* allow for comments in the middle of a line */
			cp = userid + strlen(userid) - 1;
			*cp = '\0';
			while(*--cp == ' ')
				*cp = '\0';
			gtkzephyr_make_userstruct(userid);
		}
  fclose(fp);

	} else {
		/* They have no .anyone file, don't worry about it */
	}
  g_free(filename);
}



void append_to_clist(gtkzephyr_userstruct *user, gpointer foo) {

  gint rowcount;
  gchar *rowtext[2] = {NULL,NULL};

  rowcount = gtk_clist_append(GTK_CLIST(mainclist), rowtext);
  gtk_clist_set_text(GTK_CLIST(mainclist),rowcount,1,user->userid);
  if (user->unreadmsg) {
    gchar *unreadmsg_str = g_strdup_printf("%d", user->unreadmsg);
    gtk_clist_set_text(GTK_CLIST(mainclist),rowcount,0, unreadmsg_str);
    g_free(unreadmsg_str);
  }
  gtk_clist_set_row_data(GTK_CLIST(mainclist),rowcount,user);

}



void check_user_status(gtkzephyr_userstruct *user, gpointer foo) {

	/* We don't change ->oldstatus unconditionally because we want them to
	 * disappear as soon as the unreadmsg's become read */
  
  if (user->status && !user->oldstatus) {

		gint rownumber = gtk_clist_find_row_from_data(GTK_CLIST(mainclist), user);

    if (!prefstruct->show_disconnected && rownumber == -1) {
			/* If rownumber != -1, then someone sent us a message before we detected
			 * them logging on and we added them to the clist then */
      append_to_clist(user, foo);
    }
    user->oldstatus = user->status;

  } else if (!user->status && user->oldstatus && !user->unreadmsg) {

    if (!prefstruct->show_disconnected) {
      gtk_clist_remove(GTK_CLIST(mainclist), gtk_clist_find_row_from_data(
  	    GTK_CLIST(mainclist), user));
    }
    user->oldstatus = user->status;

  }
}
   



gint update_userlist(gpointer foo) {

  gtkzephyr_update_userlist(userlist);
  userlist = g_list_sort(userlist, userlist_comparefunc);
  g_list_foreach(userlist, (GFunc) check_user_status, NULL);
  if (redraw_count < prefstruct->max_redraw) {
    gtk_clist_sort(GTK_CLIST(mainclist));
    if (++redraw_count == prefstruct->max_redraw)
      gtk_clist_freeze(GTK_CLIST(mainclist));
  }
  
  return TRUE;
}



gint redraw_clist(gpointer foo) {

  g_assert(redraw_count <= prefstruct->max_redraw);
  if (redraw_count == prefstruct->max_redraw) {
    /* gtk_clist_sort() will also thaw for us */
    gtk_clist_sort(GTK_CLIST(mainclist));
  }

  redraw_count = 0;

  return TRUE;
}



int main (int argc, char *argv[]) {

  gtk_init(&argc, &argv);

  build_main_window();
  gtkzephyr_connect();  /* should connect, subscribe, etc */
  gtkzephyr_load_prefs();
  build_user_list();
  gtkzephyr_update_userlist(userlist);
  userlist = g_list_sort(userlist, userlist_comparefunc);
  if (prefstruct->show_disconnected)
    g_list_foreach(userlist, (GFunc) append_to_clist, NULL);
  else
    g_list_foreach(userlist, (GFunc) check_user_status, NULL);
  gtk_widget_show(mainwindow);
  redraw_count = 0;
  newmsg_count = 0;

  gtk_timeout_add(prefstruct->check_messages_timeout, (GtkFunction)gtkzephyr_checknewnotices, GTK_CLIST(mainclist));

  gtk_timeout_add(prefstruct->update_ul_timeout, (GtkFunction)update_userlist, NULL);

  
  gtk_timeout_add(prefstruct->redraw_ul_timeout, (GtkFunction)redraw_clist, NULL);

  gtk_main();

  // User can't change anything, so why save it.. later on make it so it
  // save only when user changes something, and immediately when they change
  // it
  // gtkzephyr_save_prefs();
  gtkzephyr_disconnect();

  return(0);
}
