#include <zephyr/zephyr.h>
#include <gtk/gtk.h>
#include "gtkzephyr.h"
#include "userdialog.h"
#include "zcalls.h"
#include "errorbox.h"


void close_user_dialog(GtkObject *widget, gtkzephyr_userstruct *user) {

  gtk_widget_hide(user->userdialog);
  gtk_editable_delete_text(GTK_EDITABLE(gtk_object_get_data(
      GTK_OBJECT(user->newmsgclist), "text-box")), 0, -1);
  gtk_clist_clear(GTK_CLIST(user->newmsgclist));
  g_list_foreach(user->readmsgs, (GFunc) ZFreeNotice, NULL);
  g_list_free(user->readmsgs);
	gtk_editable_delete_text(GTK_EDITABLE(user->dialogsendtext), 0, -1);
  user->readmsgs = NULL;

}



void reply_callback(GtkObject *object, gtkzephyr_userstruct *user) {

  gtk_notebook_set_page(GTK_NOTEBOOK(user->notebook), 1);

}



gboolean delete_event(GtkObject *widget, GdkEvent *event, gtkzephyr_userstruct *user) {

  close_user_dialog(NULL, user);

  return TRUE;

}



void dialog_send_text(GtkWidget *widget, gpointer data) {

  gtkzephyr_userstruct *user = (gtkzephyr_userstruct *) data;
  gtk_widget_set_sensitive(GTK_WIDGET(user->dialogsendtext), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(user->dialogsendbutton), FALSE);

  gtkzephyr_send_text(user, GTK_EDITABLE(user->dialogsendtext),
      gtkzephyr_user_dialog_handle_servack);

}



void newmsgclist_row_selected(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer user_data) {

  GtkText *text = (GtkText *) gtk_object_get_data(GTK_OBJECT(clist),
      "text-box");
  gchar *buffer, **celltext;
  gint tmp = 0;
  ZNotice_t *notice = (ZNotice_t *) gtk_clist_get_row_data(clist, row);

  buffer = gtkzephyr_get_body(notice->z_message, notice->z_message_len);
  gtk_text_freeze(text);
  gtk_editable_delete_text(GTK_EDITABLE(text), 0, -1);
  if (buffer)
    gtk_editable_insert_text(GTK_EDITABLE(text), buffer, strlen(buffer), &tmp);
  gtk_text_thaw(text);
  gtk_clist_get_text(clist, row, 0, celltext);
  if (*celltext[0] == '*') {
    gtkzephyr_userstruct *user = (gtkzephyr_userstruct *)
      gtk_object_get_data(GTK_OBJECT(clist), "user-struct");
    gtk_clist_set_text(clist, row, 0, "");
    user->newmsgs = g_list_remove(user->newmsgs, notice);
    user->readmsgs = g_list_append(user->readmsgs, notice);
    gtkzephyr_remove_unreadmsg(user);
  }
  g_free(buffer);
}



void add_to_clist(ZNotice_t *notice, gtkzephyr_userstruct *user) {

  gchar* text[3];
  gint rownumber;
  time_t sec = notice->z_time.tv_sec;

  gtk_clist_freeze(GTK_CLIST(user->newmsgclist));
  DEBUG_INFO("Initializing text..\n");
  text[0] = (gchar *) &"*";
  text[1] = user->userid;
  text[2] = ctime(&sec);
	text[2][24] = '\0';
  DEBUG_INFO("Appending..\n");
  rownumber = gtk_clist_append(GTK_CLIST(user->newmsgclist), text);
  DEBUG_INFO("Setting row_data..\n");
  gtk_clist_set_row_data(GTK_CLIST(user->newmsgclist), rownumber, notice);
  DEBUG_INFO("Thawing..\n");
  gtk_clist_thaw(GTK_CLIST(user->newmsgclist));

}




void make_user_dialog(gtkzephyr_userstruct *user) {

  GtkWidget *vbox, *tab_label;
  GtkWidget *button, *hbuttonbox, *paned, *scrolledwindow;
  gchar *clisttitles[3] = {"N", "Sender","Time"};

  g_assert(user->userdialog == NULL);
  g_assert(user->notebook == NULL);
  g_assert(user->newmsgclist == NULL);
  g_assert(user->newmsgtext == NULL);
  g_assert(user->dialogsendtext == NULL);
  g_assert(user->dialogsendbutton == NULL);

  DEBUG_INFO("Dialog opened for: %s\n",user->userid);

  user->userdialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize(user->userdialog, 300, 225);
  gtk_signal_connect(GTK_OBJECT(user->userdialog), "delete_event",
      GTK_SIGNAL_FUNC(delete_event), user);
  gtk_window_set_title(GTK_WINDOW(user->userdialog), user->userid);
  gtk_container_set_border_width(GTK_CONTAINER(user->userdialog), 5);
  user->notebook = gtk_notebook_new();
  gtk_container_add(GTK_CONTAINER(user->userdialog), user->notebook);
  gtk_notebook_set_page(GTK_NOTEBOOK(user->notebook), GTK_POS_TOP);
  gtk_widget_show(user->notebook);

  /* read message tab */

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  paned = gtk_vpaned_new();
  user->newmsgclist = gtk_clist_new_with_titles(3, clisttitles);
  gtk_signal_connect(GTK_OBJECT(user->newmsgclist), "select-row",
      GTK_SIGNAL_FUNC(newmsgclist_row_selected), NULL);
  gtk_object_set_data(GTK_OBJECT(user->newmsgclist), "user-struct", user);
  gtk_widget_show(user->newmsgclist);
  scrolledwindow = gtk_scrolled_window_new(
      GTK_CLIST(user->newmsgclist)->hadjustment,
      GTK_CLIST(user->newmsgclist)->vadjustment);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), user->newmsgclist);
  gtk_widget_show(scrolledwindow);
  gtk_paned_add1(GTK_PANED(paned), scrolledwindow);
  user->newmsgtext = gtk_text_new(NULL, NULL);
  gtk_widget_show(user->newmsgtext);
  gtk_object_set_data(GTK_OBJECT(user->newmsgclist), "text-box",
      user->newmsgtext);
  scrolledwindow = gtk_scrolled_window_new(GTK_TEXT(user->newmsgtext)->hadj,
      GTK_TEXT(user->newmsgtext)->vadj);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), user->newmsgtext);
  gtk_widget_show(scrolledwindow);
  gtk_paned_add2(GTK_PANED(paned), scrolledwindow);
  gtk_widget_show(paned);
  gtk_box_pack_start(GTK_BOX(vbox), paned, TRUE, TRUE, 5);
  hbuttonbox = gtk_hbutton_box_new();
  gtk_button_box_set_child_size(GTK_BUTTON_BOX(hbuttonbox), 65, 15);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbuttonbox), 25);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
  button = gtk_button_new_with_label("Reply");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
      GTK_SIGNAL_FUNC(reply_callback), user);
  gtk_widget_show(button);
  gtk_widget_show(hbuttonbox);
  gtk_box_pack_end(GTK_BOX(hbuttonbox), button, FALSE, TRUE, 25);
  button = gtk_button_new_with_label("Close");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
      GTK_SIGNAL_FUNC(close_user_dialog), user);
  gtk_widget_show(button);
  gtk_widget_show(hbuttonbox);
  gtk_box_pack_end(GTK_BOX(hbuttonbox), button, FALSE, TRUE, 25);
  gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, FALSE, FALSE, 5);
  gtk_widget_show(vbox);
  tab_label = gtk_label_new("Read Message");
  gtk_widget_show(tab_label);
  gtk_notebook_append_page(GTK_NOTEBOOK(user->notebook), vbox, tab_label);

  /* write message tab */
  
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  gtk_widget_show(vbox);
  user->dialogsendtext = gtk_text_new(NULL, NULL);
  gtk_text_set_editable(GTK_TEXT(user->dialogsendtext), TRUE);
  gtk_widget_show(user->dialogsendtext);
  scrolledwindow = gtk_scrolled_window_new(GTK_TEXT(user->dialogsendtext)->hadj,
      GTK_TEXT(user->dialogsendtext)->vadj);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), user->dialogsendtext);
  gtk_widget_show(scrolledwindow);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
  hbuttonbox = gtk_hbutton_box_new();
  gtk_button_box_set_child_size(GTK_BUTTON_BOX(hbuttonbox), 65, 15);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbuttonbox), 25);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
  gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, FALSE, FALSE, 0);
  user->dialogsendbutton = gtk_button_new_with_label("Send");
  gtk_signal_connect(GTK_OBJECT(user->dialogsendbutton), "clicked",
      GTK_SIGNAL_FUNC(dialog_send_text), user);
  gtk_widget_show(user->dialogsendbutton);
  gtk_box_pack_end(GTK_BOX(hbuttonbox), user->dialogsendbutton, FALSE, FALSE, 5);
  button = gtk_button_new_with_label("Cancel");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
      GTK_SIGNAL_FUNC(close_user_dialog), user);
  gtk_widget_show(button);
  gtk_box_pack_end(GTK_BOX(hbuttonbox), button, FALSE, FALSE, 5);
  gtk_widget_show(hbuttonbox);
  tab_label = gtk_label_new("Write Message");
  gtk_widget_show(tab_label);
  gtk_notebook_append_page(GTK_NOTEBOOK(user->notebook), vbox, tab_label);
}



void open_user_dialog(gtkzephyr_userstruct *user) {

  if (user->notebook == NULL)
    make_user_dialog(user);

  if (user->unreadmsg && !GTK_WIDGET_VISIBLE(user->userdialog)) {
    g_list_foreach(user->newmsgs, (GFunc)add_to_clist, user);
  }

}



void gtkzephyr_open_read(GtkWidget *widget, gpointer data) {

  gtkzephyr_userstruct *user = (gtkzephyr_userstruct *)data;

  open_user_dialog(user);
  gtk_notebook_set_page(GTK_NOTEBOOK(user->notebook), 0);
  gtk_widget_show(GTK_WIDGET(user->userdialog));
  gdk_window_raise(user->userdialog->window);

}



void gtkzephyr_open_write(GtkWidget *widget, gpointer data) {

  gtkzephyr_userstruct *user = (gtkzephyr_userstruct *)data;

  open_user_dialog(user);
  gtk_notebook_set_page(GTK_NOTEBOOK(user->notebook), 1);
  gtk_widget_grab_focus(user->dialogsendtext);
  gtk_widget_show(GTK_WIDGET(user->userdialog));
  gdk_window_raise(user->userdialog->window);

}



void gtkzephyr_open_info(GtkWidget *widget, gpointer data) {

  gtkzephyr_userstruct *user = (gtkzephyr_userstruct *)data;

  open_user_dialog(user);
  gtk_notebook_set_page(GTK_NOTEBOOK(user->notebook), 2);
  gtk_widget_show(GTK_WIDGET(user->userdialog));

}


void gtkzephyr_user_dialog(gtkzephyr_userstruct *user) {

  if (user->unreadmsg)
    gtkzephyr_open_read(NULL, user);
  else
    gtkzephyr_open_write(NULL, user);

}



void gtkzephyr_user_dialog_handle_message(ZNotice_t *notice, gtkzephyr_userstruct *user) {

  
  user->newmsgs = g_list_append(user->newmsgs, notice);

  if (user->userdialog && GTK_WIDGET_VISIBLE(user->userdialog)) {
    add_to_clist(notice, user);
  }

  gtkzephyr_add_unreadmsg(user);
  
}



void gtkzephyr_user_dialog_handle_servack(ZNotice_t *notice, gtkzephyr_userstruct *user) {
  DEBUG_INFO("Entering gtkzephyr_user_dialog_handle_servack\n");
  if (notice->z_kind == SERVNAK) {
    // Don't know what to do here
  } else { /* z_kind == SERVACK */
    if (!strcmp(notice->z_message, ZSRVACK_SENT)) {
      /* Do nothing special.. (closing the window is always done) */
    } else if (!strcmp(notice->z_message, ZSRVACK_NOTSENT)) {

      gchar *error = g_strdup_printf("%s is not logged in", notice->z_recipient);
      gtkzephyr_error(error);
      g_free(error);

    } else {

      gtkzephyr_error("Unknown server response");
      gtkzephyr_handle_unknown_notice(notice);

    }
  }

  close_user_dialog(NULL, user);
  gtk_editable_delete_text(GTK_EDITABLE(user->dialogsendtext), 0, -1);
  gtk_widget_set_sensitive(GTK_WIDGET(user->dialogsendtext), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(user->dialogsendbutton), TRUE);

}
